//
//  CallKitManager.m
//  ZIMExampleDemo
//
//  Created by 武耀琳 on 2023/1/31.
//

#import "CallKitManager.h"
#import <ZPNs/ZPNs.h>
#import <Flutter/Flutter.h>
#import "ZPNsSettings.h"
#import "../utils/NSDictionary+safeInvoke.h"
#import "../utils/NSMutableDictionary+safeInvoke.h"
#import "ZPNsMethodManager.h"
#import "../utils/ZPNsUtils.h"
static CallKitManager *_callKitManager = nil;

static CXProviderConfiguration *_cxProviderConfiguration = nil;

static bool _isProduct = false;

@interface CallKitManager()

@property CXProvider *cxProvider;

@property CXCallController *cxCallController;

@property NSString *callID;

@property(nonatomic,assign) BOOL isEnableVoIP;

@property(nonatomic,strong) NSData *voIPToken;

@property unsigned int seq;

@property NSMutableDictionary<NSString *,CXAction *> *actionMap;


@end

@implementation CallKitManager

-(BOOL)isContainInitConfig{
    if(_cxProviderConfiguration == nil || _cxProviderConfiguration == NULL || [_cxProviderConfiguration isEqual:[NSNull null]]){
        return false;
    }else{
        return true;
    }
}

-(void)setInitConfig:(NSDictionary *)configMap{
    NSString *localizedName = [configMap safeObjectForKey:@"localizedName"];
    NSNumber *supportVideo = [configMap safeObjectForKey:@"supportsVideo"];
    NSNumber *maximumCallsPerCallGroup = [configMap safeObjectForKey:@"maximumCallsPerCallGroup"];
    NSNumber *maximumCallGroups = [configMap safeObjectForKey:@"maximumCallGroups"];
    NSArray<NSNumber *> *supportedHandleTypes = [configMap safeObjectForKey:@"supportedHandleTypes"];
    NSString *iconTemplateImageName = [configMap safeObjectForKey:@"iconTemplateImageName"];
    
    _cxProviderConfiguration = [[CXProviderConfiguration alloc] initWithLocalizedName:localizedName];
    if(supportVideo != nil)_cxProviderConfiguration.supportsVideo = [supportVideo boolValue];
    if(maximumCallsPerCallGroup != nil)_cxProviderConfiguration.maximumCallsPerCallGroup = [maximumCallsPerCallGroup unsignedIntegerValue];
    if(maximumCallGroups != nil)_cxProviderConfiguration.maximumCallGroups = [maximumCallGroups unsignedIntegerValue];
    if(supportedHandleTypes != nil)_cxProviderConfiguration.supportedHandleTypes = [[NSSet alloc] initWithArray:supportedHandleTypes];
    if(iconTemplateImageName != nil)_cxProviderConfiguration.iconTemplateImageData = UIImagePNGRepresentation([UIImage imageNamed:iconTemplateImageName]);
}

+(CallKitManager *)getInstance{
    if(!_callKitManager){
        @synchronized (self) {
            if(!_callKitManager){
                _callKitManager = [[CallKitManager alloc] init];
                _callKitManager.actionMap = [[NSMutableDictionary alloc] init];
                _callKitManager.seq = 0;
            }
        }
    }
    return _callKitManager;
}

+(void)isProduct:(bool) isProduct{
    _isProduct = isProduct;
}

-(void)enableVoIP{
    self.isEnableVoIP = YES;
    if(ZPNsSettings.event != nil){
        [ZPNsSettings clearHoldNomalResultArr];
    }
    if(self.voIPToken != nil){
        [[ZPNs shared] setVoipToken:self.voIPToken isProduct:_isProduct];
    }
    
}

-(void)isSetEvent:(BOOL)isSetEvent{
    if(isSetEvent == NO){
        return;
    }
    if(self.isEnableVoIP == YES){
        [ZPNsSettings clearHoldNomalResultArr];
    }
}

-(void)reportIncomingCallWithTitle:(NSDictionary *)updateMap UUID:(NSString *)uuidString completion:(void (^)(NSError *_Nullable error))completion{
    CXCallUpdate *update = [self oCXCallUpdate:updateMap];
    NSUUID *uuid = [[NSUUID alloc] initWithUUIDString:uuidString];
    if(![ZPNsUtils isNotNull:uuid]){
        NSError *error = [NSError errorWithDomain:@"com.zego.zego_zpns" code:1 userInfo:@{NSLocalizedDescriptionKey: @"Assign nsuuid failed. invalid uuid strings."}];
        [ZPNsUtils wirteCustomLog:[NSString stringWithFormat:@"Assign nsuuid failed. invalid strings.uuidString:%@",uuidString]];
        completion(error);
        return;
    }
    if(_callKitManager.cxProvider == nil){
        if(_cxProviderConfiguration){
            _callKitManager.cxProvider = [[CXProvider alloc] initWithConfiguration:_cxProviderConfiguration];
            [_callKitManager.cxProvider setDelegate:_callKitManager queue:dispatch_get_main_queue()];
        }else{
            NSError *error = [NSError errorWithDomain:@"com.zego.zego_zpns" code:1 userInfo:@{NSLocalizedDescriptionKey: @"Please invoke setInitConfiguration before invoke reportIncomingCall."}];
            [ZPNsUtils wirteCustomLog:[NSString stringWithFormat:@"Please invoke setInitConfiguration before invoke reportIncomingCall.uuidString:%@",uuidString]];
            completion(error);
            return;
        }
    }
    if(_callKitManager.cxCallController == nil){
        _callKitManager.cxCallController = [[CXCallController alloc] initWithQueue:dispatch_get_main_queue()];
    }
    
    [self.cxProvider reportNewIncomingCallWithUUID:uuid update:update completion:completion];
}

- (void)reportCallEndedWithUUID:(NSString *)uuidString reason:(CXCallEndedReason)endedReason{
    [self.cxProvider reportCallWithUUID:[[NSUUID alloc] initWithUUIDString:uuidString] endedAtDate:nil reason:endedReason];
}

-(void)reportCallUpdateWithUUID:(NSDictionary *)updateMap UUID:(NSString *)uuidString{
    CXCallUpdate *update = [self oCXCallUpdate:updateMap];
    NSUUID *uuid = [[NSUUID alloc] initWithUUIDString:uuidString];
    [self.cxProvider reportCallWithUUID:uuid updated:update];
}

-(void)reportOutgoingCallWithUUID:(NSString *)uuidString{
    NSUUID *uuid = [[NSUUID alloc] initWithUUIDString:uuidString];
    [self.cxProvider reportOutgoingCallWithUUID:uuid connectedAtDate:nil];
}
#pragma mark - voIP delegate

- (void)pushRegistry:(nonnull PKPushRegistry *)registry didUpdatePushCredentials:(nonnull PKPushCredentials *)pushCredentials forType:(nonnull PKPushType)type {
    NSString *voipToken = [ZPNsUtils getHexStringForData:[pushCredentials token] ];
    [ZPNsUtils wirteCustomLog:[NSString stringWithFormat:@"setVoipToken, voip token:%@,isProduct:%@",voipToken,_isProduct?@"true":@"false"]];
    NSLog(@"voip isProduct:%d",_isProduct);
    self.voIPToken = [pushCredentials token];
    if(self.isEnableVoIP){
        [[ZPNs shared] setVoipToken:[pushCredentials token] isProduct:_isProduct];
    }
    NSLog(@"voip token %@",voipToken);
}

- (void)pushRegistry:(PKPushRegistry *)registry didReceiveIncomingPushWithPayload:(PKPushPayload *)payload forType:(PKPushType)type withCompletionHandler:(void (^)(void))completion{
    [ZPNsUtils
          wirteCustomLog:[NSString stringWithFormat:@"didReceiveIncomingPushWithPayload in comming."]];
    NSUUID *uuid = [[NSUUID alloc] init];
    NSDictionary *resultDic = @{@"method":@"didReceiveIncomingPushWithPayload",@"payload":payload.dictionaryPayload,@"uuidString":uuid.UUIDString};
    
    NSString *title = nil;
    NSNumber *voIPHasVideo = nil;
    NSString *voIPHandleValue = nil;
    NSNumber *voIPHandleType = nil;
    
    // 取 zego 字段
    NSDictionary *zegoDic = [payload.dictionaryPayload safeObjectForKey:@"zego"];
    if([ZPNsUtils isNotNull:zegoDic]){
        
        voIPHasVideo = [zegoDic safeObjectForKey:@"voip_has_video"];
        voIPHandleValue = [zegoDic safeObjectForKey:@"voip_handle_value"];
        voIPHandleType = [zegoDic safeObjectForKey:@"voip_handle_type"];
    }
    
    if(_callKitManager.cxProvider == nil){
        _callKitManager.cxProvider = [[CXProvider alloc] initWithConfiguration:_cxProviderConfiguration];
        [_callKitManager.cxProvider setDelegate:_callKitManager queue:dispatch_get_main_queue()];
    }
    if(_callKitManager.cxCallController == nil){
        _callKitManager.cxCallController = [[CXCallController alloc] initWithQueue:dispatch_get_main_queue()];
    }
    

    
    NSObject *aps = [[payload dictionaryPayload] safeObjectForKey:@"aps"];
    
    // 判断 alert 为 string 还是 map, string 直接赋值给 title, map 取其中的 title ,如果对应的值为空则不赋值
    if([ZPNsUtils isNotNull:aps] && [aps isKindOfClass:[NSDictionary class]]){
        NSDictionary *apsDic = (NSDictionary *)aps;
        NSObject *alertObj = [apsDic safeObjectForKey:@"alert"];
        if([ZPNsUtils isNotNull:alertObj] && [alertObj isKindOfClass:[NSDictionary class]]){
            NSString *tmpTitle = [((NSDictionary *)alertObj) safeObjectForKey:@"title"];
            if([ZPNsUtils isNotNull:tmpTitle]){
                title = tmpTitle;
            }
        }else if([ZPNsUtils isNotNull:alertObj] && [alertObj isKindOfClass:[NSString class]]){
            title = (NSString *)alertObj;
        }
    }
    
    CXCallUpdate *update = [[CXCallUpdate alloc] init];
    if([ZPNsUtils isNotNull:voIPHasVideo]){
        update.hasVideo = [voIPHasVideo boolValue];
    }
    if([ZPNsUtils isNotNull:voIPHandleType] && [ZPNsUtils isNotNull:voIPHandleValue]){
        switch ((CXHandleType)[voIPHandleType integerValue]) {
            case CXHandleTypeGeneric:
            case CXHandleTypePhoneNumber:
            case CXHandleTypeEmailAddress:{
                update.remoteHandle =[[CXHandle alloc] initWithType:(CXHandleType)[voIPHandleType integerValue] value:voIPHandleValue];
                break;
            }
            default:
                [ZPNsUtils wirteCustomLog:[NSString stringWithFormat:@"voIP handle type enum is index out of range.vaule:%ld",[voIPHandleType integerValue]]];
                break;
        }
        
    }
    if([ZPNsUtils isNotNull:title]){
        update.localizedCallerName = title;
    }
    
    
    [self.cxProvider reportNewIncomingCallWithUUID:uuid update:update completion:^(NSError * _Nullable error) {
        NSMutableString *log = [NSMutableString stringWithFormat:@"didReceiveIncomingPush completion. title:%@ ",
                        title];
        if([payload.dictionaryPayload objectForKey:@"call_id"]){
            [log appendFormat:@"call id:%@ ",[payload.dictionaryPayload safeObjectForKey:@"call_id"]];
        }
        if(!(error == nil ||error == NULL || [error isEqual:[NSNull null]])){
            [log appendFormat:@"error:%@",error];
        }
        [ZPNsUtils
              wirteCustomLog:log];
        completion();
    }];
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:resultDic];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] didReceiveIncomingPushWithPayload UUID:%@",uuid.UUIDString]];
        ZPNsSettings.event(resultDic);
    }
}

//当接收到呼叫重置时 调用的函数，这个函数必须被实现，其不需做任何逻辑，只用来重置状态
- (void)providerDidReset:(CXProvider *)provider{
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"providerDidReset"}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] providerDidReset."]];
        ZPNsSettings.event(@{@"method":@"providerDidReset"});
    }
}

//呼叫开始时回调
- (void)providerDidBegin:(CXProvider *)provider{
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"providerDidBegin"}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] providerDidBegin."]];
        ZPNsSettings.event(@{@"method":@"providerDidBegin"});
    }
}
//音频会话激活状态的回调
- (void)provider:(CXProvider *)provider didActivateAudioSession:(AVAudioSession *)audioSession{
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"didActivateAudioSession"}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] didActivateAudioSession."]];
        ZPNsSettings.event(@{@"method":@"didActivateAudioSession"});
    }
}
//音频会话停用的回调
- (void)provider:(CXProvider *)provider didDeactivateAudioSession:(AVAudioSession *)audioSession{
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"didDeactivateAudioSession"}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] didDeactivateAudioSession."]];
        ZPNsSettings.event(@{@"method":@"didDeactivateAudioSession"});
    }
}
//行为超时的回调
- (void)provider:(CXProvider *)provider timedOutPerformingAction:(CXAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"timedOutPerformingAction"}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] timedOutPerformingAction."]];
        ZPNsSettings.event(@{@"method":@"timedOutPerformingAction",@"seq":seq});
    }
}
//有事务被提交时调用
//如果返回YES 则表示事务被捕获处理 后面的回调都不会调用 如果返回NO 则表示事务不被捕获，会回调后面的函数
- (BOOL)provider:(CXProvider *)provider executeTransaction:(CXTransaction *)transaction{
    return NO;
}
//点击开始按钮的回调
- (void)provider:(CXProvider *)provider performStartCallAction:(CXStartCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"performStartCallAction",@"seq":seq}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] performStartCallAction.seq:%@",seq]];
        ZPNsSettings.event(@{@"method":@"performStartCallAction",@"seq":seq});
    }
}
//点击接听按钮的回调
- (void)provider:(CXProvider *)provider performAnswerCallAction:(CXAnswerCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"performAnswerCallAction",@"seq":seq}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] performAnswerCallAction.seq:%@",seq]];
        ZPNsSettings.event(@{@"method":@"performAnswerCallAction",@"seq":seq});
    }
}
//点击结束按钮的回调
- (void)provider:(CXProvider *)provider performEndCallAction:(CXEndCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"performEndCallAction",@"seq":seq}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] performEndCallAction.seq:%@",seq]];
        ZPNsSettings.event(@{@"method":@"performEndCallAction",@"seq":seq});
    }
}
//点击保持通话按钮的回调
- (void)provider:(CXProvider *)provider performSetHeldCallAction:(CXSetHeldCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"performSetHeldCallAction",@"seq":seq}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] performSetHeldCallAction.seq:%@",seq]];
        ZPNsSettings.event(@{@"method":@"performSetHeldCallAction",@"seq":seq});
    }
}
//点击静音按钮的回调
- (void)provider:(CXProvider *)provider performSetMutedCallAction:(CXSetMutedCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"performSetMutedCallAction",@"seq":seq,@"muted":[NSNumber numberWithBool:action.muted]}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] performSetMutedCallAction.seq:%@",seq]];
        ZPNsSettings.event(@{@"method":@"performSetMutedCallAction",@"seq":seq,@"muted":[NSNumber numberWithBool:action.muted]});
    }
}
//点击组按钮的回调
- (void)provider:(CXProvider *)provider performSetGroupCallAction:(CXSetGroupCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"performSetGroupCallAction",@"seq":seq}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] performSetMutedCallAction.seq:%@",seq]];
        ZPNsSettings.event(@{@"method":@"performSetGroupCallAction",@"seq":seq});
    }
}
//DTMF功能回调
- (void)provider:(CXProvider *)provider performPlayDTMFCallAction:(CXPlayDTMFCallAction *)action{
    NSString *seq = [self generateSeqStr];
    [self.actionMap safeSetObject:action forKey:seq];
    if(ZPNsSettings.event == nil){
        [ZPNsSettings addDicToHoldNomalResultArr:@{@"method":@"performPlayDTMFCallAction",@"seq":seq}];
    }else{
        [ZPNsUtils
              wirteCustomLog:[NSString stringWithFormat:@"[CALLBACK] performPlayDTMFCallAction.seq:%@",seq]];
        ZPNsSettings.event(@{@"method":@"performPlayDTMFCallAction",@"seq":seq});
    }
}

-(void)ActionFail:(NSString *)seq{
    CXAction *action = [_actionMap safeObjectForKey:seq];
    if([ZPNsUtils isNotNull:action]){
        [action fail];
        [_actionMap removeObjectForKey:seq];
        [ZPNsUtils wirteCustomLog:[NSString stringWithFormat:@"report [action fail] success.seq:%@",seq]];
    }else{
        [ZPNsUtils wirteCustomLog:[NSString stringWithFormat:@"report [action fail] failed. target action not at the action map. seq:%@",seq]];
    }
    
}

-(void)ActionFullFill:(NSString *)seq{
    CXAction *action = [_actionMap safeObjectForKey:seq];
    if([ZPNsUtils isNotNull:action]){
        [action fulfill];
        [_actionMap removeObjectForKey:seq];
        [ZPNsUtils wirteCustomLog:[NSString stringWithFormat:@"report [action fulfill] success.seq:%@",seq]];
    }else{
        [ZPNsUtils wirteCustomLog:[NSString stringWithFormat:@"report [action fulfill] failed. target action not at the action map. seq:%@",seq]];
    }
}

#pragma mark - Utils
-(CXCallUpdate *) oCXCallUpdate:(NSDictionary *)updateMap{
    CXCallUpdate *update = [[CXCallUpdate alloc] init];
    if([updateMap safeObjectForKey:@"supportsDTMF"])update.supportsDTMF = [[updateMap safeObjectForKey:@"supportsDTMF"] boolValue];
    if([updateMap safeObjectForKey:@"supportsHolding"])update.supportsHolding = [[updateMap safeObjectForKey:@"supportsHolding"] boolValue];
    if([updateMap safeObjectForKey:@"supportsGrouping"])update.supportsGrouping = [[updateMap safeObjectForKey:@"supportsGrouping"] boolValue];
    if([updateMap safeObjectForKey:@"supportsUngrouping"])update.supportsUngrouping = [[updateMap safeObjectForKey:@"supportsUngrouping"] boolValue];
    if([updateMap safeObjectForKey:@"hasVideo"])update.hasVideo = [[updateMap safeObjectForKey:@"hasVideo"] boolValue];
    if([updateMap safeObjectForKey:@"remoteHandle"]){
        NSDictionary *remoteHandleMap = [updateMap safeObjectForKey:@"remoteHandle"];
        update.remoteHandle = [[CXHandle alloc] initWithType:[[remoteHandleMap safeObjectForKey:@"type"] intValue] value:[remoteHandleMap safeObjectForKey:@"value"]];
    }
    if([updateMap safeObjectForKey:@"localizedCallerName"])update.localizedCallerName = [updateMap safeObjectForKey:@"localizedCallerName"];
    return update;
}

-(NSString *)generateSeqStr{
    ++_seq;
    NSString *seqStr = [NSString stringWithFormat:@"native%u",_seq];
    return seqStr;
}

@end
